@@ -9,7 +9,7 @@ require DynaLoader;
 use vars qw($VERSION @ISA $__import @preload $pid);
 @ISA = qw(DynaLoader);
 sub dl_load_flags { 0x00 }
-$VERSION = '1.67_1';
+$VERSION = '1.68_1';
 $pid = $$;
 bootstrap Prima $VERSION;
 unless ( UNIVERSAL::can('Prima', 'init')) {
@@ -79,6 +79,7 @@ sub run
 			};
 			eval {
 				$::application-> go if $::application-> alive;
+				$got_exception = 0;
 			};
 		}
 		last unless defined $@ && $got_exception;
@@ -140,6 +141,8 @@ Prima - a perl graphic toolkit
 
 	run Prima;
 
+See more screenshots at L<http://prima.eu.org/big-picture> .
+
 =head1 DESCRIPTION
 
 The toolkit is combined from two basic set of classes - core and external. The
@@ -407,6 +410,10 @@ L<Prima::Spinner> - spinner animation
 
 L<Prima::TextView> - rich text browser widget
 
+L<Prima::Widget::Date> - standard date picker widget
+
+L<Prima::Widget::Time> - standard time input widget
+
 =item Standard dialogs
 
 L<Prima::Dialog::ColorDialog> - color selection facilities
@@ -552,7 +559,7 @@ viewer C<podview> ( see File/Run/prima-class ).
 Copyright 1997-2003 The Protein Laboratory, University of Copenhagen. All
 rights reserved.
 
-Copyright 2004-2020 Dmitry Karasik. All rights reserved. 
+Copyright 2004-2023 Dmitry Karasik. All rights reserved. 
 
 This program is distributed under the BSD License.
 
