@@ -14,11 +14,10 @@ package Prima::Buttons;
 use strict;
 use warnings;
 use Carp;
-use Prima qw(IntUtils StdBitmap);
+use Prima qw(StdBitmap);
 
 package Prima::AbstractButton;
-use vars qw(@ISA);
-@ISA = qw(Prima::Widget Prima::MouseScroller);
+use base qw(Prima::Widget Prima::Widget::MouseScroller);
 
 {
 my %RNT = (
@@ -90,6 +89,15 @@ sub init
 	return %profile;
 }
 
+sub skin
+{
+	return $_[0]->SUPER::skin unless $#_;
+	my $self = shift;
+	$self->SUPER::skin($_[1]);
+	$self->buffered(1) if $self->SUPER::skin eq 'flat';
+	$self->repaint;
+}
+
 sub cancel_transaction
 {
 	my $self = $_[0];
@@ -425,20 +433,12 @@ sub init
 	return %profile;
 }
 
-sub on_paint
+sub paint_classic
 {
-	my ($self,$canvas)  = @_;
-	my @clr  = ( $self-> color, $self-> backColor);
-	@clr = ( $self-> hiliteColor, $self-> hiliteBackColor)
-		if $self-> { default};
-	$clr[1] = $self-> prelight_color($clr[1]) if $self->{hilite} && $self-> enabled;
-	@clr = ( $self-> disabledColor, $self-> disabledBackColor)
-		if !$self-> enabled;
-	my @size = $canvas-> size;
-
+	my ( $self, $canvas, $w, $h, @clr) = @_;
 	my @fbar = $self-> {default} ?
-		( 1, 1, $size[0] - 2, $size[1] - 2):
-		( 0, 0, $size[0] - 1, $size[1] - 1);
+		( 1, 1, $w - 2, $h - 2):
+		( 0, 0, $w - 1, $h - 1);
 	if ( !$self-> {flat} || $self-> {hilite}) {
 		$self-> rect_bevel( $canvas, @fbar,
 			fill => $self->transparent ? undef : $self-> new_gradient(
@@ -455,11 +455,58 @@ sub on_paint
 	}
 	if ( $self-> {default}) {
 		$canvas-> color( cl::Black);
-		$canvas-> rectangle( 0, 0, $size[0]-1, $size[1]-1);
+		$canvas-> rectangle( 0, 0, $w-1, $h-1);
 	}
+}
+
+sub paint_flat
+{
+	my ( $self, $canvas, $w, $h, @clr) = @_;
+	$canvas-> graphic_context( sub {
+		$canvas-> color( $self-> owner-> backColor );
+		$canvas-> bar( 0, 0, $w, $h);
+		$canvas-> backColor( $clr[1]);
+		$canvas-> color( $clr[0]);
+		my $lw = 1;
+		$lw++ if $self->{hilite} && $self->enabled;
+		$lw++ if $self->{default};
+		$canvas-> antialias(1);
+		$canvas-> lineWidth( $lw );
+		$canvas-> new_path-> round_rect(
+			0 + $lw/2, 0 + $lw/2, $w - $lw/2 - 1, $h - $lw/2 - 1, 
+			$self-> font-> height
+		)-> fill_stroke;
+	}) unless $self-> transparent;
+}
+
+sub on_paint
+{
+	my ($self,$canvas)  = @_;
+	my @clr  = ( $self-> color, $self-> backColor);
+	my $skin = $self->skin;
+
+	my $boldify;
+	if ($self-> { default}) {
+		if ( $skin eq 'flat') {
+			@clr = ( $self-> hiliteBackColor, $self-> hiliteColor);
+			$boldify++;
+		} else {
+			@clr = ( $self-> hiliteColor, $self-> hiliteBackColor);
+		}
+	}
+	$clr[1] = $self-> prelight_color($clr[1]) if $self->{hilite} && $self-> enabled;
+	@clr = ( $self-> disabledColor, $self-> disabledBackColor)
+		if !$self-> enabled;
+	my @size = $canvas-> size;
 
 	my $shift  = $self-> {checked} ? 1 : 0;
+	if ( $skin eq 'flat') {
+		$self->paint_flat($canvas, @size, @clr);
+	} else {
+		$self->paint_classic($canvas, @size, @clr);
+	}
 	$shift += $self-> {pressed} ? 2 : 0;
+
 	my $capOk = length($self-> text) > 0;
 	my ( $fw, $fh) = $capOk ? $self-> caption_box($canvas) : ( 0, 0);
 	my ( $textAtX, $textAtY);
@@ -576,7 +623,11 @@ CAPTION:
 		$textAtY = ( $size[1] - $fh) / 2 - $shift;
 	}
 	$canvas-> color( $clr[0]);
-	$self-> draw_caption( $canvas, $textAtX, $textAtY) if $capOk;
+	if ($capOk) {
+		$canvas-> font-> style( $canvas-> font-> style  | fs::Bold )
+			if $boldify;
+		$self-> draw_caption( $canvas, $textAtX, $textAtY);
+	}
 	$canvas-> rect_focus( 4, 4, $size[0] - 5, $size[1] - 5 ) if !$capOk && $self-> focused;
 }
 
@@ -920,6 +971,102 @@ sub profile_default
 	}
 }
 
+sub paint_classic
+{
+	my ( $self, $canvas, $w, $h, @clr) = @_;
+
+	my ( $image, $imNo);
+	if ( $self-> { checked}) {
+		$imNo = $self-> { pressed} ? 3 : 2;
+	} else {
+		$imNo = $self-> { pressed} ? 1 : 0;
+	};
+	my $xStart;
+	$image = $images[ $imNo];
+	my @c3d  = ( $self-> light3DColor, $self-> dark3DColor);
+
+	if ( $image) {
+		$canvas-> put_image( 0, ( $h - $image-> height) / 2, $image);
+		$xStart = $image-> width;
+	} else {
+		my $s = $::application->uiScaling;
+		$xStart = $s * 16;
+		push ( @c3d, shift @c3d)
+			if $self-> { pressed};
+		$canvas-> rect3d( 1, ( $h - $s*14) / 2, $s*15, ( $h + $s*14) / 2, 1,
+			@c3d, $clr[ 1]);
+		if ( $self-> { checked}) {
+			my $at = $self-> { pressed} ? 1 : 0;
+			$canvas-> color( cl::Black);
+			$canvas-> lineWidth( 2);
+			my $yStart = ( $h - $s*14) / 2;
+			$canvas-> line(
+				$at + $s*4, $yStart - $at +  $s*8,
+				$at + $s*5 , $yStart - $at + $s*3
+			);
+			$canvas-> line(
+				$at + $s*5 , $yStart - $at + $s*3,
+				$at + $s*12, $yStart - $at + $s*12
+			);
+			$canvas-> lineWidth( 0);
+		}
+	}
+	return $xStart;
+}
+
+sub paint_flat_background
+{
+	my ( undef, $path, $height ) = @_;
+	my $s = $::application->uiScaling;
+	$path-> round_rect(
+		1,       ( $height - $s*14) / 2,
+		$s * 15, ( $height + $s*14) / 2,
+		$s * 7
+	);
+	return $s * 15;
+}
+
+sub paint_flat_checkmark
+{
+	my ( undef, $path, $height ) = @_;
+	my $s = $::application->uiScaling;
+	my $yStart = ( $height - $s*14) / 2;
+	$path-> line(
+		$s*4,  $yStart + $s*8,
+		$s*5 , $yStart + $s*3,
+		$s*5 , $yStart + $s*3,
+		$s*12, $yStart + $s*12
+	);
+}
+
+sub paint_flat
+{
+	my ( $self, $canvas, $w, $h, @clr) = @_;
+
+	my $s = $::application->uiScaling;
+	my $xStart = $s * 16;
+
+	$canvas-> graphic_context( sub {
+		$canvas-> antialias(1);
+		$canvas-> lineWidth( 1 );
+		$canvas-> color($clr[0]);
+		my $path1 = $canvas->new_path;
+		$self-> paint_flat_background( $path1, $h);
+		$path1->stroke;
+
+		if ( $self-> { checked} || $self->{ pressed } ) {
+			my $at = $self-> { pressed} ? 1 : 0;
+			$canvas-> color( cl::Black);
+			$canvas-> lineWidth( 2);
+			$canvas-> translate( $at, -$at);
+			my $path2 = $canvas->new_path;
+			$self-> paint_flat_checkmark( $path2, $h);
+			$path2->stroke;
+		}
+	});
+	return $xStart;
+}
+
 sub on_paint
 {
 	my ($self,$canvas) = @_;
@@ -941,41 +1088,11 @@ sub on_paint
 		$canvas-> bar( 0, 0, @size);
 	}
 
-	my ( $image, $imNo);
-	if ( $self-> { checked}) {
-		$imNo = $self-> { pressed} ? 3 : 2;
-	} else {
-		$imNo = $self-> { pressed} ? 1 : 0;
-	};
 	my $xStart;
-	$image = $images[ $imNo];
-	my @c3d  = ( $self-> light3DColor, $self-> dark3DColor);
-
-	if ( $image) {
-		$canvas-> put_image( 0, ( $size[1] - $image-> height) / 2, $image);
-		$xStart = $image-> width;
+	if ( $self->skin eq 'flat') {
+		$xStart = $self->paint_flat( $canvas, @size, @clr);
 	} else {
-		my $s = $::application->uiScaling;
-		$xStart = $s * 16;
-		push ( @c3d, shift @c3d)
-			if $self-> { pressed};
-		$canvas-> rect3d( 1, ( $size[1] - $s*14) / 2, $s*15, ( $size[1] + $s*14) / 2, 1,
-			@c3d, $clr[ 1]);
-		if ( $self-> { checked}) {
-			my $at = $self-> { pressed} ? 1 : 0;
-			$canvas-> color( cl::Black);
-			$canvas-> lineWidth( 2);
-			my $yStart = ( $size[1] - $s*14) / 2;
-			$canvas-> line(
-				$at + $s*4, $yStart - $at +  $s*8,
-				$at + $s*5 , $yStart - $at + $s*3
-			);
-			$canvas-> line(
-				$at + $s*5 , $yStart - $at + $s*3,
-				$at + $s*12, $yStart - $at + $s*12
-			);
-			$canvas-> lineWidth( 0);
-		}
+		$xStart = $self->paint_classic( $canvas, @size, @clr);
 	}
 
 	$canvas-> color( $clr[ 0]);
@@ -995,27 +1112,9 @@ sub profile_default
 	return $def;
 }
 
-sub on_paint
+sub paint_classic
 {
-	my ($self,$canvas) = @_;
-	my @clr;
-	if ( $self-> enabled) {
-		if ( $self-> focused) {
-			@clr = ($self-> hiliteColor, $self-> hiliteBackColor);
-		} else {
-			@clr = ($self-> color, $self-> backColor);
-		}
-		$clr[1] = $self-> prelight_color($clr[1]) if $self->{hilite} && $self-> enabled;
-	} else {
-		@clr = ($self-> disabledColor, $self-> disabledBackColor);
-	}
-
-	my @size = $canvas-> size;
-	unless ( $self-> transparent) {
-		$canvas-> color( $clr[ 1]);
-		$canvas-> bar( 0, 0, @size);
-	}
-
+	my ( $self, $canvas, $w, $h, @clr) = @_;
 	my ( $image, $imNo);
 	if ( $self-> { checked}) {
 		$imNo = $self-> { pressed} ? 7 : 6;
@@ -1026,12 +1125,12 @@ sub on_paint
 	my $xStart;
 	$image = $images[ $imNo];
 	if ( $image) {
-		$canvas-> put_image( 0, ( $size[1] - $image-> height) / 2, $image);
+		$canvas-> put_image( 0, ( $h - $image-> height) / 2, $image);
 		$xStart = $image-> width;
 	} else {
 		my $s = $::application->uiScaling;
 		$xStart = $s * 16;
-		my $y = ( $size[1] - $s * 16) / 2;
+		my $y = ( $h - $s * 16) / 2;
 		my @xs = map { $s * $_ } ( 0, 8, 16, 8);
 		my @ys = map { $s * $_ } ( 8, 16, 8, 0);
 		for ( @ys) {$_+=$y};
@@ -1068,6 +1167,60 @@ sub on_paint
 			$canvas-> fillpoly( [ $s*6, $y+$s*8, $s*8, $y+$s*10, $s*10, $y+$s*8, $s*8, $y+$s*6]);
 		}
 	}
+}
+
+sub paint_flat
+{
+	my ( $self, $canvas, $w, $h, @clr) = @_;
+
+	my $s = $::application->uiScaling;
+	my $xStart = $s * 16;
+
+	$canvas-> graphic_context( sub {
+		$canvas-> antialias(1);
+		$canvas-> lineWidth( 1 );
+		$canvas-> color($clr[0]);
+
+		my $center = $h / 2;
+		my $x = 1 + $s * 7.5;
+		$canvas-> ellipse( $x, $center, $s * 15, $s * 15);
+
+		if ( $self-> { checked} || $self->{ pressed } ) {
+			my $at = $self-> {pressed} ? 1 : 0;
+			$canvas-> fill_ellipse( $x + $at, $center - $at, $s * 8, $s * 8);
+		}
+	});
+	return $xStart;
+}
+
+sub on_paint
+{
+	my ($self,$canvas) = @_;
+	my @clr;
+	if ( $self-> enabled) {
+		if ( $self-> focused) {
+			@clr = ($self-> hiliteColor, $self-> hiliteBackColor);
+		} else {
+			@clr = ($self-> color, $self-> backColor);
+		}
+		$clr[1] = $self-> prelight_color($clr[1]) if $self->{hilite} && $self-> enabled;
+	} else {
+		@clr = ($self-> disabledColor, $self-> disabledBackColor);
+	}
+
+	my @size = $canvas-> size;
+	unless ( $self-> transparent) {
+		$canvas-> color( $clr[ 1]);
+		$canvas-> bar( 0, 0, @size);
+	}
+
+	my $xStart;
+	if ( $self->skin eq 'flat') {
+		$xStart = $self->paint_flat( $canvas, @size, @clr);
+	} else {
+		$xStart = $self->paint_classic( $canvas, @size, @clr);
+	}
+
 	$canvas-> color( $clr[ 0]);
 	my ( $fw, $fh) = $self-> caption_box( $canvas);
 	$self-> draw_caption( $canvas, $xStart * 1.5, ( $size[1] - $fh) / 2 );
@@ -1138,11 +1291,27 @@ sub profile_default
 {
 	return {
 		%{$_[ 0]-> SUPER::profile_default},
+		border             => 1,
 		ownerBackColor     => 1,
 		autoEnableChildren => 1,
 	}
 }
 
+sub init
+{
+	my $self = shift;
+	my %profile = $self-> SUPER::init(@_);
+	$self-> {border}  = $profile{border};
+	return %profile;
+}
+
+sub border
+{
+	return $_[0]->{border} unless $#_;
+	$_[0]->{border} = $_[1];
+	$_[0]->repaint;
+}
+
 sub on_radioclick
 {
 	my ($me,$rd) = @_;
@@ -1165,10 +1334,17 @@ sub on_paint
 		$canvas-> bar( 0, 0, @size);
 	}
 	my $fh = $canvas-> font-> height;
-	$canvas-> color( $self-> light3DColor);
-	$canvas-> rectangle( 1, 0, $size[0] - 1, $size[1] - $fh / 2 - 2);
-	$canvas-> color( $self-> dark3DColor);
-	$canvas-> rectangle( 0, 1, $size[0] - 2, $size[1] - $fh / 2 - 1);
+	if ( $self->{border}) {
+		if ( $self->skin eq 'flat') {
+			$canvas-> color( $self-> dark3DColor);
+			$canvas-> rectangle( 0, 0, $size[0] - 1, $size[1] - $fh / 2 - 1);
+		} else {
+			$canvas-> color( $self-> light3DColor);
+			$canvas-> rectangle( 1, 0, $size[0] - 1, $size[1] - $fh / 2 - 2);
+			$canvas-> color( $self-> dark3DColor);
+			$canvas-> rectangle( 0, 1, $size[0] - 2, $size[1] - $fh / 2 - 1);
+		}
+	}
 	my $c = $self->text;
 	if ( length( $c) > 0) {
 		$c = $self-> text_shape($c, skip_if_simple => 1) || $c;
@@ -1692,6 +1868,10 @@ to disable this functionality.
 
 =over
 
+=item border BOOLEAN
+
+If set (default), draws a border along the widget boundaries
+
 =item index INTEGER
 
 Checks the child radio button with C<index>. The indexing is
@@ -1734,7 +1914,7 @@ Dmitry Karasik, E<lt>dmitry@karasik.eu.orgE<gt>.
 
 =head1 SEE ALSO
 
-L<Prima>, L<Prima::Widget>, L<Prima::Window>, L<Prima::IntUtils>,
+L<Prima>, L<Prima::Widget>, L<Prima::Window>, 
 L<Prima::Drawable::Metafile>,
 L<Prima::StdBitmap>, F<examples/buttons.pl>, F<examples/buttons2.pl>.
 
