@@ -1,11 +1,7 @@
 use strict;
 use warnings;
 
-use Prima;
-use Prima::Classes;
-use Prima::Label;
-use Prima::ComboBox;
-use Prima::Sliders;
+use Prima qw(Classes Label ComboBox Sliders Utils);
 
 package Prima::Calendar;
 use vars qw(@ISA @non_locale_months @days_in_months $OB_format);
@@ -57,7 +53,7 @@ sub init
 	$self-> reset_days;
 
 	my $right_offset = $self-> font-> width * 9;
-	$self-> insert( ComboBox => 
+	$self-> insert( ComboBox =>
 		origin   => [ 5, $h - $fh * 2 - 10 ],
 		size     => [ $w - $right_offset - 15, $fh + 4],
 		name     => 'Month',
@@ -67,15 +63,15 @@ sub init
 		delegations => [ 'Change' ],
 	);
 
-	$self-> insert( Label => 
+	$self-> insert( Label =>
 		origin => [ 5, $h - $fh - 4],
 		size   => [ $w - $right_offset - 15, $fh + 2],
 		text   => '~Month',
 		focusLink => $self-> Month,
 		growMode => gm::GrowHiX | gm::GrowLoY,
 	);
-	
-	$self-> insert( SpinEdit => 
+
+	$self-> insert( SpinEdit =>
 		origin => [ $w - $right_offset - 5, $h - $fh * 2 - 10 ],
 		size   => [ $right_offset, $fh + 4],
 		name   => 'Year',
@@ -85,7 +81,7 @@ sub init
 		delegations => [ 'Change' ],
 	);
 
-	$self-> insert( Label => 
+	$self-> insert( Label =>
 		origin => [ $w - $right_offset - 5, $h - $fh - 4],
 		size   => [ $right_offset, $fh + 2],
 		text   => '~Year',
@@ -99,14 +95,14 @@ sub init
 		name        => 'Day',
 		selectable  => 1,
 		current     => 1,
-		delegations => [ qw( 
+		delegations => [ qw(
 			Paint MouseDown MouseMove MouseUp MouseWheel MouseLeave
 			KeyDown Size FontChanged Enter Leave
 		)],
 		growMode    => gm::Client,
 	);
-	
-	$self-> insert( Label => 
+
+	$self-> insert( Label =>
 		origin => [ 5, $h - $fh * 3 - 15],
 		size   => [ $w - 10, $fh + 2],
 		text   => '~Day',
@@ -114,22 +110,38 @@ sub init
 		growMode => gm::GrowHiX | gm::GrowLoY,
 	);
 	$self-> $_($profile{$_}) for qw( date useLocale firstDayOfWeek);
+	$self-> reset_geometry;
 }
 
+sub reset_geometry
+{
+	my $self = shift;
+	my $font = $self->font;
+	my ( $fw, $fh ) = ($font->width, $font-> height);
+	$self-> geomSize( $fw * 35, $fh * 20 );
+}
+
+sub on_fontchanged { shift-> reset_geometry }
 
 sub can_use_locale
 {
 	return $posix_state if defined $posix_state;
 	undef $@;
-	eval "use POSIX q(strftime);"; 
+	eval "use POSIX q(strftime);";
 	return $posix_state = 1 unless $@;
 	return $posix_state = 0;
 }
 
+sub locale_str($$$)
+{
+	my ( $format, $day, $month ) = @_;
+	Prima::Utils::local2sv(POSIX::strftime ( $format, 0, 0, 0, $day, $month, 0 ));
+}
+
 sub month2str
 {
 	return $non_locale_months[$_[1]] unless $_[0]-> {useLocale};
-	return POSIX::strftime ( "%B", 0, 0, 0, 1, $_[1], 0 );
+	return locale_str "%B", 1, $_[1];
 }
 
 sub make_months
@@ -143,9 +155,7 @@ sub make_months
 			POSIX::strftime ( "%OB", 0, 0, 0, 1, 1, 0 )
 		) ? '%B' : '%OB';
 	}
-	return [ map {
-		POSIX::strftime ( $OB_format, 0, 0, 0, 1, $_, 0 )
-	} 0 .. 11 ];
+	return [ map { locale_str $OB_format, 1, $_ } 0 .. 11 ];
 }
 
 sub day_of_week
@@ -156,7 +166,7 @@ sub day_of_week
 	$useFirstDayOfWeek = 1 unless defined $useFirstDayOfWeek;
 
 	if ( $month < 3) {
-		$month += 10; 
+		$month += 10;
 		$year--;
 	} else {
 		$month -= 2;
@@ -164,8 +174,8 @@ sub day_of_week
 	my $century = int($year / 100);
 	$year %= 100;
 	my $dow = ( int(( 26 * $month - 2) / 10) + $day + $year + int($year / 4) +
-		int($century / 4) - ( 2 * $century) - 
-		(( $useFirstDayOfWeek ? 1 : 0) * $self-> {firstDayOfWeek}) + 7) 
+		int($century / 4) - ( 2 * $century) -
+		(( $useFirstDayOfWeek ? 1 : 0) * $self-> {firstDayOfWeek}) + 7)
 		% 7;
 	return ($dow < 0) ? $dow + 7 : $dow;
 }
@@ -175,8 +185,8 @@ sub reset_days
 	my $self = $_[0];
 	my $dow = $self-> {firstDayOfWeek};
 	$self-> {days} = $self-> {useLocale} ?
-	[ map { strftime("%a", 0, 0, 0, $_, 0, 0) } 0 .. 6 ] :
-	[ qw( Sun Mon Tue Wed Thu Fri Sat ) ];
+		[ map { locale_str "%a", $_, 0 } 0 .. 6 ] :
+		[ qw( Sun Mon Tue Wed Thu Fri Sat ) ];
 	push @{$self-> {days}}, splice( @{$self-> {days}}, 0, $dow) if $dow;
 }
 
@@ -190,34 +200,39 @@ sub useLocale
 	$self-> Month-> text( $self-> Month-> List-> get_item_text( $self-> Month-> focusedItem));
 	$self-> reset_days;
 	my $day = $self-> Day;
-	$self-> day_reset( $day, $day-> size); 
+	$self-> day_reset( $day, $day-> size);
 	$day-> repaint;
 }
 
-sub Day_Paint 
+sub Day_Paint
 {
 	my ( $owner, $self, $canvas) = @_;
 
 	my @sz = $self-> size;
-	$canvas-> set( color => $self-> disabledColor, backColor => $self-> disabledBackColor) 
+	$canvas-> set( color => $self-> disabledColor, backColor => $self-> disabledBackColor)
 		unless $self-> enabled;
-	$canvas-> rect3d( 0, 0, $sz[0]-1, $sz[1]-1, 2, 
-		$self-> dark3DColor, $self-> light3DColor, $self-> backColor);
+	if ( $owner-> skin eq 'flat') {
+		$canvas-> clear( Prima::rect->new(@sz)->inclusive );
+	} else {
+		$canvas-> rect3d( Prima::rect->new(@sz)->inclusive, 2,
+			$self-> dark3DColor, $self-> light3DColor, $self-> backColor);
+	}
 
-	my @zs = ( $self-> {X}, $self-> {Y}, $self-> {CX1}, $self-> {CY}); 
+	my @zs = ( $self-> {X}, $self-> {Y}, $self-> {CX1}, $self-> {CY});
 	my $i;
 	my $c = $canvas-> color;
-	my $b = $canvas-> backColor;
-	$canvas-> color( int((($b & 0xff0000)>>16)*0.8)<<16 |
-                  int((($b & 0xff00)>>8)*0.8)<<8 |
-                  int((($b & 0xff))*0.8));
+	$canvas-> color( $self-> prelight_color($canvas-> backColor));
 	$canvas-> bar( 2, $sz[1] - $zs[1] - 3, $sz[0] - 3, $sz[1] - 3);
 	$canvas-> color($c);
-	$canvas-> clipRect( 2, 2, $sz[0] - 3, $sz[1] - 3);
+	$canvas-> clipRect( Prima::rect->new(@sz)->shrink(2)->inclusive );
+	my $fdo = $owner-> firstDayOfWeek ? 6 : 0;
 	for ( $i = 0; $i < 7; $i++) {
-		$canvas-> text_out_bidi( $owner-> {days}-> [$i], 
-			$i * $zs[0] + $self-> {CX3}, $sz[1]-$zs[1]+$zs[3],
+		my $tw = $canvas->get_text_width( $owner-> {days}-> [$i] );
+		$canvas-> color( 0xc00000 ) if $i == $fdo;
+		$canvas-> text_shape_out( $owner-> {days}-> [$i],
+			$i * $zs[0] + ($self->{X} - $tw)/2, $sz[1]-$zs[1]+$zs[3],
 		);
+		$canvas-> color( $c ) if $i == $fdo;
 	}
 
 	my ( $d, $m, $y) = @{$owner-> {date}};
@@ -231,19 +246,22 @@ sub Day_Paint
 			my $bk = ($d == $i) ? cl::Hilite : cl::Back;
 			$bk = $self->prelight_color($bk) if $prelight == $i;
 			$canvas-> color($bk);
-			$canvas-> bar( 
-				$dow * $zs[0] + 2, $y - $zs[3], 
+			$canvas-> bar(
+				$dow * $zs[0] + 2, $y - $zs[3],
 				( 1 + $dow) * $zs[0] - 1, $y - $zs[3] + $zs[1] - 1
 			);
 			$canvas-> color(( $d == $i) ? cl::HiliteText : cl::Fore);
-			$canvas-> text_out_bidi( $i + 1, $dow * $zs[0] + $zs[2], $y);
-			$canvas-> rect_focus( 
-				$dow * $zs[0] + 2, $y - $zs[3], 
+			$canvas-> color( 0xc00000 ) if $d != $i && $dow == $fdo;
+			$canvas-> text_shape_out( $i + 1, $dow * $zs[0] + $zs[2], $y);
+			$canvas-> rect_focus(
+				$dow * $zs[0] + 2, $y - $zs[3],
 				( 1 + $dow) * $zs[0] - 1, $y - $zs[3] + $zs[1] - 1
 			) if $d == $i && $self-> focused;
 			$canvas-> color( $c);
 		} else {
-			$canvas-> text_out_bidi( $i + 1, $dow * $zs[0] + $zs[2], $y);
+			$canvas-> color( 0xc00000 ) if $dow == $fdo;
+			$canvas-> text_shape_out( $i + 1, $dow * $zs[0] + $zs[2], $y);
+			$canvas-> color( $c ) if $dow == $fdo;
 		}
 		$zs[2] = $self-> {CX2} if $i == 8;
 		next unless $dow++ == 6;
@@ -281,11 +299,26 @@ sub xy2day
 	my @zs = ( $widget-> {X}, $widget-> {Y});
 	my $v = $days_in_months[ $month] + (((( $year % 4) == 0) && ( $month == 1)) ? 1 : 0);
 	my @sz = $widget-> size;
-	$day = (int(($sz[1] - $y - 2) / $zs[1]) - 1) * 7 + 
+	$day = (int(($sz[1] - $y - 2) / $zs[1]) - 1) * 7 +
 		int(($x - 2) / $zs[0]) - $self-> day_of_week( 1, $month, $year) + 1;
 	return ($day <= 0 || $day > $v) ? undef : $day;
 }
 
+sub day2xy
+{
+	my ($self, $day) = @_;
+	my (undef, $month, $year) = @{$self-> {date}};
+	my $v = $days_in_months[ $month] + (((( $year % 4) == 0) && ( $month == 1)) ? 1 : 0);
+	return if $day <= 0 || $day > $v;
+	my $widget = $self->Day;
+	my @zs = ( $widget-> {X}, $widget-> {Y});
+	my @sz = $widget-> size;
+
+	$day = $day - 1 + $self->day_of_week(1, $month, $year);
+	my ($x, $y) =  ($zs[0] * ($day % 7) + 2, $sz[1] - 2 - $zs[1] * (2 + int($day / 7)) - 1);
+	return $x, $y, $x + $zs[0] + 1, $y + $zs[1];
+}
+
 sub Day_MouseDown
 {
 	my ( $owner, $self, $btn, $mod, $x, $y) = @_;
@@ -295,6 +328,7 @@ sub Day_MouseDown
 	$self-> clear_event;
 	return unless defined $day;
 	$self-> {mouseTransaction} = 1;
+	delete $self->{prelight};
 	$owner-> date( $day, $month, $year);
 }
 
@@ -305,8 +339,11 @@ sub Day_MouseMove
 	my $day = $owner->xy2day($x,$y);
 	unless ($self-> {mouseTransaction}) {
 		if (( $self->{prelight} // -1 ) != ( $day // -1 )) {
+			my $p = $self->{prelight};
 			$self->{prelight} = $day;
-			$self-> invalidate_rect( 2, 2, $self-> width - 3, $self-> height - $self-> {Y} - 3);
+			my $r = Prima::rect->new( defined($p) ? $owner->day2xy($p) : ());
+			$r = $r->union( Prima::rect->new( $owner->day2xy($day) )) if defined $day;
+			$self->invalidate_rect( @$r ) unless $r->is_empty;
 		}
 		return;
 	}
@@ -324,8 +361,10 @@ sub Day_MouseUp
 
 sub Day_MouseLeave
 {
-	my $self = $_[1];
-	$self->repaint if delete $self->{prelight};
+	my ($owner,$self) = @_;
+	my $p = delete $self->{prelight} or return;
+	my @p = $owner-> day2xy($p) or return;
+	$self-> invalidate_rect(@p);
 }
 
 sub Day_MouseWheel
@@ -362,6 +401,7 @@ sub day_reset
 		$self-> get_text_width('28'),
 		$self-> get_text_width( $owner-> {days}-> [0])
 	);
+	$x3 += $x1/2;
 	$y = $self-> font-> height;
 	$self-> end_paint_info;
 	$self-> {X} = $x2 if $self-> {X} < $x2;
@@ -369,14 +409,13 @@ sub day_reset
 	$self-> {Y} = $y if $self-> {Y} < $y;
 	$self-> {CX1} = int(( $self-> {X} - $x1 ) / 2) + 4;
 	$self-> {CX2} = int(( $self-> {X} - $x2 ) / 2) + 4;
-	$self-> {CX3} = int(( $self-> {X} - $x3 ) / 2) + 4;
 	$self-> {CY} = int(( $self-> {Y} - $y ) / 2);
 }
 
 sub Day_Size
 {
 	my ( $owner, $self, $ox, $oy, $x, $y) = @_;
-	$owner-> day_reset( $self, $x, $y); 
+	$owner-> day_reset( $self, $x, $y);
 }
 
 sub Day_FontChanged
@@ -409,7 +448,7 @@ sub date
 	$year = 0 if $year < 0;
 	$year = 199 if $year > 199;
 	$day = 1 if $day < 1;
-	my $v = $days_in_months[ $month] + 
+	my $v = $days_in_months[ $month] +
 		(((( $year % 4) == 0) && ( $month == 1)) ? 1 : 0);
 	$day = $v if $day > $v;
 	my @od = @{$self-> {date}};
@@ -417,8 +456,15 @@ sub date
 	$self-> {date} = [ $day, $month, $year ];
 	$self-> Year-> value( $year + 1900);
 	$self-> Month-> focusedItem( $month);
-	$day = $self-> Day;
-	$day-> invalidate_rect( 2, 2, $day-> width - 3, $day-> height - $day-> {Y} - 3);
+	my $widget = $self->Day;
+	if ( $month == $od[1] && $year == $od[2] ) {
+		my $r = Prima::rect->new( $self->day2xy($od[0]) );
+		$r = $r->union( Prima::rect->new(  $self->day2xy($day) ));
+		$widget->invalidate_rect( @$r ) unless $r->is_empty;
+	} else {
+		delete $widget->{prelight};
+		$widget->invalidate_rect( 2, 2, $widget-> width - 3, $widget-> height - $widget-> {Y} - 3);
+	}
 	$self-> notify(q(Change));
 }
 
@@ -464,7 +510,7 @@ sub firstDayOfWeek
 	return if $dow == $self-> {firstDayOfWeek};
 	$self-> {firstDayOfWeek} = $dow;
 	$self-> reset_days;
-	$self-> Day-> repaint; 
+	$self-> Day-> repaint;
 }
 
 1;
@@ -477,21 +523,27 @@ Prima::Calendar - standard calendar widget
 
 =head1 SYNOPSIS
 
-	use Prima::Calendar;
+	use Prima qw(Calendar Application);
 	my $cal = Prima::Calendar-> create(
 		useLocale => 1,
+		size      => [ 150, 150 ],
 		onChange  => sub {
 			print $_[0]-> date_as_string, "\n";
 		},
 	);
 	$cal-> date_from_time( localtime );
 	$cal-> month( 5);
+	run Prima;
+
+=for podview <img src="calendar.gif">
+
+=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/calendar.gif">
 
 =head1 DESCRIPTION
 
 Provides interactive selection of date between 1900 and 2099 years.
 The main property, L<date>, is a three-integer array, day, month, and year,
-in the format of perl localtime ( see L<perlfunc/localtime> ) - 
+in the format of perl localtime ( see L<perlfunc/localtime> ) -
 day can be in range from 1 to 31,month from 0 to 11, year from 0 to 199.
 
 =head1 API
