@@ -395,6 +395,48 @@ sub _debug
 	print STDERR "\n";
 }
 
+package Prima::Matrix;
+
+sub new
+{
+	my $class = shift;
+	my $self = { canvas => shift };
+	bless( $self, $class);
+	return $self;
+}
+
+sub get          { shift->{canvas}->get_matrix(@_) }
+sub set          { shift->{canvas}->set_matrix(@_) }
+sub is_identity  { shift->{canvas}->get_matrix->is_identity }
+sub reset        { shift->{canvas}->set_matrix( $Prima::matrix::identity ) }
+sub canvas       { $#_ ? $_[0]->{canvas} : $_[0]->{canvas} = $_[1] }
+
+sub save         { push @{ $_[0]->{stack} }, $_[0]->{canvas}->get_matrix }
+sub restore      { @{ $_[0]->{stack} } ? $_[0]->{canvas}->set_matrix( pop @{ $_[0]->{stack} } ) : croak("stack underflow") }
+
+eval <<PROP for qw( A B C D X Y );
+	sub $_
+	{
+		return shift->{canvas}->get_matrix->$_() unless \$#_;
+		my \$self = shift;
+		my \$m    = \$self->{canvas}->get_matrix();
+		\$m->$_(\@_);
+		\$self->{canvas}->set_matrix(\$m);
+		return \$self;
+	}
+PROP
+
+eval <<METH for qw( identity translate scale shear rotate transform inverse_transform);
+	sub $_
+	{
+		my \$self = shift;
+		my \$m    = \$self->{canvas}->get_matrix();
+		\$m->$_(\@_);
+		\$self->{canvas}->set_matrix(\$m);
+		return \$self;
+	}
+METH
+
 package Prima::Component;
 use vars qw(@ISA);
 @ISA = qw(Prima::Object);
@@ -416,9 +458,10 @@ sub profile_default
 {
 	my $def = $_[ 0]-> SUPER::profile_default;
 	my %prf = (
-		name        => ref $_[ 0],
-		owner       => $::application,
-		delegations => undef,
+		name            => ref $_[ 0],
+		owner           => $::application,
+		delegations     => undef,
+		deepChildLookup => 0,
 	);
 	@$def{keys %prf} = values %prf;
 	return $def;
@@ -471,8 +514,6 @@ sub get_notify_sub
 	return undef;
 }
 
-sub AUTOLOAD_DEEP_LOOKUP { $#_ ? $_[0]->{_autoload_deep_lookup} = $_[1] : $_[0]->{_autoload_deep_lookup} }
-
 sub AUTOLOAD
 {
 	no strict;
@@ -481,7 +522,7 @@ sub AUTOLOAD
 	Carp::confess "There is no such thing as \"$expectedMethod\"\n"
 		if scalar(@_) or not ref $self;
 	my ($componentName) = $expectedMethod =~ /::([^:]+)$/;
-	my $component = $self-> bring( $componentName, $self->{_autoload_deep_lookup} ? 1000 : 0);
+	my $component = $self-> bring( $componentName, $self->deepChildLookup ? 1000 : 0);
 	Carp::confess("Unknown widget or method \"$expectedMethod\"")
 		unless $component && ref($component);
 	return $component;
@@ -517,12 +558,6 @@ sub profile_default
 	return $def;
 }
 
-sub profile_check_in
-{
-	my ( $self, $p, $default) = @_;
-	$p->{fd} = fileno($p->{file}) if exists $p->{file} && ! exists $p->{fd};
-}
-
 package Prima::Clipboard;
 use vars qw(@ISA);
 @ISA = qw(Prima::Component);
@@ -685,6 +720,15 @@ sub font
 		)
 }
 
+sub matrix
+{
+	return Prima::Matrix->new($_[0]) unless $#_;
+	$_[0]-> set_matrix( $#_ > 1 ?
+		[@_[1 .. $#_]] :
+		((ref($_[1]) eq 'Prima::Matrix') ? $_[1]->get : $_[1])
+	);
+}
+
 sub put_image
 {
 	$_[0]-> put_image_indirect(
@@ -737,10 +781,10 @@ sub graphic_context
 
 sub translate
 {
-	return @{$_[0]->matrix}[4,5] unless $#_;
-	my $m = $_[0]->matrix;
+	return @{$_[0]->get_matrix}[4,5] unless $#_;
+	my $m = $_[0]->get_matrix;
 	@{$m}[4,5] = ref($_[1]) ? @{$_[1]} : @_[1,2];
-	$_[0]->matrix($m);
+	$_[0]->set_matrix($m);
 }
 
 sub lineTail  { $#_ ? $_[0]->lineEndIndex(lei::Only | lei::LineTail , $_[1])  : $_[0]->lineEndIndex(lei::Only | lei::LineTail ) }
@@ -1157,6 +1201,7 @@ sub notification_types { return \%RNT; }
 	ownerFont         => 1,
 	ownerHint         => 1,
 	ownerShowHint     => 1,
+	ownerSkin         => 1,
 	ownerPalette      => 1,
 	packInfo          => undef,
 	packPropagate     => 1,
@@ -1182,6 +1227,7 @@ sub notification_types { return \%RNT; }
 	selectingButtons  => mb::Left,
 	shape             => undef,
 	showHint          => 1,
+	skin              => undef,
 	syncPaint         => 0,
 	tabOrder          => -1,
 	tabStop           => 1,
@@ -1334,12 +1380,8 @@ sub profile_check_in
 				@default_font_box = ( $f-> { width}, $f-> { height});
 			}
 			my @a = @default_font_box;
-			$p-> {left}    *= $a[0] / $d[0] if exists $p-> {left};
-			$p-> {right}   *= $a[0] / $d[0] if exists $p-> {right};
-			$p-> {top}     *= $a[1] / $d[1] if exists $p-> {top};
-			$p-> {bottom}  *= $a[1] / $d[1] if exists $p-> {bottom};
-			$p-> {width}   *= $a[0] / $d[0] if exists $p-> {width};
-			$p-> {height}  *= $a[1] / $d[1] if exists $p-> {height};
+			$p->{$_} = int($p->{$_} * $a[0] / $d[0] + .5) for grep { exists $p->{$_} } qw(left right width);
+			$p->{$_} = int($p->{$_} * $a[1] / $d[1] + .5) for grep { exists $p->{$_} } qw(top bottom height);
 		}
 	} else {
 		$p-> {designScale} = [0,0];
@@ -1433,6 +1475,7 @@ sub popupDisabledColor    { return shift-> popupColorIndex( ci::DisabledText, @_
 sub popupHiliteColor      { return shift-> popupColorIndex( ci::HiliteText  , @_)};
 sub popupDark3DColor      { return shift-> popupColorIndex( ci::Dark3DColor , @_)};
 sub popupLight3DColor     { return shift-> popupColorIndex( ci::Light3DColor, @_)};
+sub skins                 {qw(classic flat)}
 
 sub x_centered       {($#_)?$_[0]-> set_centered(1,0)      :$_[0]-> raise_wo("x_centered"); }
 sub y_centered       {($#_)?$_[0]-> set_centered(0,1)      :$_[0]-> raise_wo("y_centered"); }
@@ -1608,11 +1651,19 @@ sub rect_bevel
 	if ( $opt{concave}) {
 		push @c3d, 0x404040, $c3d[0];
 	} elsif ( $opt{panel}) {
-		@c3d = ( 0x404040, $self-> disabledBackColor, $c3d[0], $c3d[1]);
+		@c3d = (
+			0x404040, cl::blend( $self->map_color($self->backColor), $self->map_color($c3d[1]), 0.5),
+			$c3d[0], $c3d[1]
+		);
 	} else {
 		push @c3d, $c3d[1], 0x404040;
 	}
 
+	if ( $opt{focused}) {
+		my $hilite = $self->map_color(cl::Hilite);
+		$_ = cl::blend( $self->map_color($_), $hilite, 0.5) for @c3d;
+	}
+
 	$fill = $fill->clone( widgetClass => $self->widgetClass ) if $fill && ref($fill);
 
 	my $hw = int( $width / 2);
@@ -1778,6 +1829,7 @@ sub begin_drag
 	my ($ret, $counterpart) = $self->dnd_start($actions, !$opt{preview});
 	if ( $self->alive ) {
 		if ( $ret == dnd::None && $opt{preview} ) {
+			require Prima::Utils;
 			my @npp = $::application->pointerPos;
 			$npp[1] -= $opt{preview}->height;
 			my $paint_flag = 0;
@@ -1804,11 +1856,11 @@ sub begin_drag
 			my @targ = map { $_ / 2 } $flyback->size;
 			while (abs( $npp[0] - $opp[0]) > $targ[0] || abs($npp[1] - $opp[1]) > $targ[1]) {
 				@npp = map { $npp[$_] * 0.8 + $opp[$_] * 0.2 } 0, 1;
-				my $max_wait = 10;
-				$::application->yield while !$paint_flag && $max_wait--;
+				Prima::Utils::wait( \$paint_flag, 5 );
 				last unless $flyback;
 				$paint_flag = 0;
-				CORE::select(undef, undef, undef, 0.02);
+				Prima::Utils::sleep(0.02);
+				last unless $flyback;
 				$flyback->origin(@npp);
 				$flyback->bring_to_front;
 			}
@@ -2263,8 +2315,10 @@ sub profile_default
 		ownerBackColor => 0,
 		ownerFont      => 0,
 		ownerShowHint  => 0,
+		ownerSkin      => 0,
 		ownerPalette   => 0,
 		showHint       => 1,
+		skin           => 'flat',
 		hintClass      => 'Prima::HintWidget',
 		hintColor      => cl::Black,
 		hintBackColor  => 0xffff80,
