@@ -55,8 +55,9 @@ $capture_mode = (Prima::Application-> get_system_info-> {apc} == apc::Unix);
 sub profile_default
 {
 	my $f = $_[ 0]-> get_default_font;
+	my $e = Prima::InputLine->profile_default;
 	return {
-		%{Prima::InputLine-> profile_default},
+		%$e,
 		%{Prima::ListBox-> profile_default},
 		%{$_[ 0]-> SUPER::profile_default},
 		style          => cs::Simple,
@@ -66,7 +67,7 @@ sub profile_default
 		autoHScroll    => 0,
 		autoVScroll    => 1,
 		listVisible    => 0,
-		editHeight     => $f-> {height} + 2,
+		editHeight     => $e-> {height},
 		listHeight     => $::application-> uiScaling * 100,
 		ownerBackColor => 1,
 		selectable     => 0,
@@ -104,7 +105,7 @@ sub profile_check_in
 		my $fh = exists $p-> {font}-> {height} ?
 			$p-> {font}-> {height} :
 			$default-> {font}-> {height};
-		$p-> { editHeight } = $fh + 2
+		$p-> { editHeight } = $fh + 6
 			unless exists $p-> {editHeight };
 	}
 	$p-> {autoHScroll} = 0 if exists $p-> {hScroll};
@@ -143,10 +144,9 @@ sub init
 		%{$profile{editProfile}},
 	);
 
-	if ( $self->{autoHeight} && $self->{style} != cs::Simple) {
-		$self->check_auto_size;
-		( $w, $h) = ( $self-> size);
-		$eh = $self-> geomHeight;
+	if ( $self->{autoHeight} ) {
+		$eh = $self-> {edit}-> default_geom_height
+			if $self->{edit}->can('default_geom_height');
 		$self->{edit}->set( height => $eh, bottom => $h - $eh );
 	}
 
@@ -183,6 +183,7 @@ sub init
 		delegations    => $profile{buttonDelegations},
 		%{$profile{buttonProfile}},
 	);
+	$self->check_auto_size;
 
 	$self-> visible( $visible);
 	return %profile;
@@ -318,6 +319,25 @@ sub Button_MouseLeave
 
 sub Button_MouseUp { $_[1]-> capture(0); }
 
+sub fix_triangle
+{
+	my @spot = map { int($_ + .5) } @_;
+	my $dx = $spot[4] - $spot[0];
+	my $dy = $spot[1] - $spot[3];
+	if ($dx % 2) {
+		$spot[2] = $spot[0] + ($dx - 1) / 2;
+		$spot[4]--;
+		$dx--;
+	}
+	if ( $dx == 2 ) {
+		$spot[4]++;
+		$spot[0]--;
+		$dx += 2;
+	}
+	$spot[3]-- if $dy < $dx / 2;
+	return @spot;
+}
+
 sub Button_Paint
 {
 	my ( $owner, $self, $canvas) = @_;
@@ -326,17 +346,45 @@ sub Button_Paint
 	my @clr    = $ena ?
 		( $self-> color, $self-> backColor) :
 		( $self-> disabledColor, $self-> disabledBackColor);
-	$clr[1] = $self->prelight_color($clr[1]) if $self->{prelight};
+	my $rc;
 	my $lv = $owner-> listVisible;
-	my ( $rc, $lc) = ( $self-> light3DColor, $self-> dark3DColor);
-	( $rc, $lc) = ( $lc, $rc) if $lv;
-	$canvas-> rect3d( 0, 0, $w-1, $h-1, 1, $rc, $lc, $clr[1]);
-	if ( $ena) {
-		$canvas-> color( $rc);
-		$canvas-> fillpoly([ 5, $h * 0.6 - 1, $w - 4, $h * 0.6 - 1, $w/2 + 1, $h * 0.4 - 1]);
+	my @triangle = fix_triangle( 4, $h * 0.6, $w/2, $h * 0.4, $w - 4, $h * 0.6 );
+	if ( $self-> skin eq 'flat') {
+		my $hbc = $self-> hiliteBackColor;
+		$canvas-> graphic_context( sub {
+			$canvas-> color( $self-> dark3DColor ); 
+			$canvas-> rectangle(0, 0, $w-1, $h-1);
+			my $c = cl::blend(
+				$self->map_color( $hbc ),
+				$self->map_color( $clr[1] ),
+				$self->{prelight} ? 0.33 : 0.66
+			);
+			$canvas-> color( $c );
+			$canvas-> bar( 1, 1, $w-2, $h-2);
+		});
+		$clr[0] = $self->hiliteColor;
+		$canvas-> translate(1,-1) if $lv;
+	} else {
+		$clr[1] = $self->prelight_color($clr[1]) if $self->{prelight};
+		$self-> rect_bevel(
+			$canvas, 0, 0, $w-1, $h-1,
+			fill => $self-> new_gradient(
+				palette  => [ $self-> dark3DColor, $clr[1], $self-> light3DColor ],
+				spline   => [0,0.5,1,0.5],
+				vertical => 0,
+			),
+			width => 2
+		);
+		$rc = $lv ? $self-> dark3DColor : $self-> light3DColor;
+		if ( $ena) {
+			$canvas-> color( $rc);
+			$canvas-> translate(1,-1);
+			$canvas-> fillpoly(\@triangle);
+			$canvas-> translate(0,0);
+		}
 	}
 	$canvas-> color( $clr[0]);
-	$canvas-> fillpoly([ 4, $h * 0.6, $w - 5, $h * 0.6, $w/2, $h * 0.4]);
+	$canvas-> fillpoly(\@triangle);
 }
 
 sub Button_Enable  { $_[1]-> repaint }
@@ -375,7 +423,9 @@ sub InputLine_KeyDown
 		( $key != kb::Space) &&
 		( $key != kb::Backspace)
 	) {
-		return if $key == kb::Tab || $key == kb::BackTab || $key == kb::NoKey;
+		return if $key == kb::Tab || $key == kb::BackTab || $key == kb::NoKey ||
+			(($key & kb::ShiftL & kb::ShiftR) != 0);
+		return unless $code;
 		$edit-> {incline} = '';
 		$self-> listVisible(1), $edit-> clear_event
 			if $key == kb::Down && $_[0]-> {style} != cs::Simple;
@@ -521,6 +571,12 @@ sub set_style
 	return if $self-> {style} == $style;
 	my $decr = (( $self-> {style} == cs::Simple) || ( $style == cs::Simple)) ? 1 : 0;
 	$self-> {style} = $style;
+	my $eh = $self->editHeight;
+	if ( $self->{autoHeight} ) {
+		$self->check_auto_size;
+		$eh = $self-> {edit}-> default_geom_height
+			if $self->{edit}->can('default_geom_height');
+	}
 	if ( $style == cs::Simple) {
 		$self-> set(
 			height=> $self-> height + $self-> listHeight,
@@ -530,7 +586,7 @@ sub set_style
 			visible    => 1,
 			origin     => [ 0, 0],
 			width      => $self-> width,
-			height     => $self-> height - $self-> editHeight ,
+			height     => $self-> height - $eh,
 			clipOwner  => 1,
 			selectable => 1,
 		);
@@ -548,15 +604,15 @@ sub set_style
 		$self-> listVisible( 0);
 	}
 	$self-> {edit}-> set(
-		bottom => $self-> height - $self-> editHeight ,
+		bottom => $self-> height - $eh,
 		width  => $self-> { edit}-> width + $::application-> uiScaling * DefButtonX * $decr *
 			(( $style == cs::Simple) ? 1 : -1),
-		height => $self-> editHeight ,
+		height => $eh,
 		dndAware => (( $style == cs::DropDown) ? 'Text' : 0 ),
 	);
 	$self-> {button}-> set(
-		bottom => $self-> height - $self-> editHeight ,
-		height => $self-> editHeight ,
+		bottom => $self-> height - $eh,
+		height => $eh,
 		visible=> $style != cs::Simple,
 	);
 	if ( $style == cs::DropDownList) {
@@ -659,7 +715,7 @@ Prima::ComboBox - standard combo box widget
 
 	run Prima;
 
-=for podview <img src="combo.gif" cut=1>
+=for podview <img src="combo.gif">
 
 =for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/combo.gif">
 
@@ -872,7 +928,7 @@ Dmitry Karasik, E<lt>dmitry@karasik.eu.orgE<gt>.
 
 =head1 SEE ALSO
 
-L<Prima>, L<Prima::InputLine>, L<Prima::Lists>, L<Prima::ColorDialog>, L<Prima::FileDialog>,
+L<Prima>, L<Prima::InputLine>, L<Prima::Lists>, L<Prima::Dialog::ColorDialog>, L<Prima::Dialog::FileDialog>,
 F<examples/listbox.pl>.
 
 =cut
