@@ -194,22 +194,25 @@ sub on_drawitem
 
 	my ( $iconWidth, $iconHeight) = $icon ?
 		( $icon-> width/$glyphs, $icon-> height) : ( 0, 0);
-	if ( $type == ROOT || $type == NODE) {
-		$x = $left + 2 + $indent + $iconWidth / 2;
-		$x = int( $x);
-		$y = ($top + $bottom) / 2;
-		$canvas-> line( $x, $bottom, $x, $y);
-	}
 
-	if ( $type != ROOT && $type != ROOT_ONLY) {
-		$x = $left + 2 + $prevIndent + $iconWidth / 2;
-		$x = int( $x);
-		$x2 = $left + 2 + $indent + $iconWidth / 2;
-		$x2 = int( $x2);
-		$y = ($top + $bottom) / 2;
-		$canvas-> line( $x, $y, $x2, $y);
-		$canvas-> line( $x, $y, $x, $top);
-		$canvas-> line( $x, $y, $x, $bottom) if $type == LEAF;
+	if ( $self->skin ne 'flat') {
+		if ( $type == ROOT || $type == NODE) {
+			$x = $left + 2 + $indent + $iconWidth / 2;
+			$x = int( $x);
+			$y = ($top + $bottom) / 2;
+			$canvas-> line( $x, $bottom, $x, $y);
+		}
+
+		if ( $type != ROOT && $type != ROOT_ONLY) {
+			$x = $left + 2 + $prevIndent + $iconWidth / 2;
+			$x = int( $x);
+			$x2 = $left + 2 + $indent + $iconWidth / 2;
+			$x2 = int( $x2);
+			$y = ($top + $bottom) / 2;
+			$canvas-> line( $x, $y, $x2, $y);
+			$canvas-> line( $x, $y, $x, $top);
+			$canvas-> line( $x, $y, $x, $bottom) if $type == LEAF;
+		}
 	}
 
 	$canvas-> put_image_indirect ( $icon,
@@ -819,7 +822,7 @@ sub init
 	$self-> insert( InputLine =>
 		name      => 'Name',
 		origin    => [ 14, 343],
-		size      => [ 245, 25],
+		size      => [ 245, 26],
 		text      => $profile{fileName},
 		maxLen    => 32768,
 		delegations => [qw(KeyDown)],
@@ -1351,10 +1354,9 @@ sub Open_Click
 				return;
 			}
 			return if !$self-> {openMode} && $self-> {overwritePrompt} && (
-					Prima::MsgBox::message_box( $self-> text,
-					"File $_ already exists. Overwrite?",
-					mb::OKCancel|mb::Warning) != mb::OK);
-
+				Prima::MsgBox::message_box( $self-> text,
+				"File $_ already exists. Overwrite?",
+				mb::OKCancel|mb::Warning) != mb::OK);
 		} else {
 			my ( $dirTo, $fileTo) = ( m{^(.*[:/\\])([^:\\/]*)$});
 			$dirTo = '.', $fileTo = $_ unless defined $dirTo;
@@ -1364,7 +1366,7 @@ sub Open_Click
 					mb::OKCancel|mb::Information
 				) != mb::OK);
 				my $file;
-				if ( open $file, ">$_") {
+				if ( open $file, ">", $_) {
 					close $file;
 				} else {
 					Prima::MsgBox::message_box( $self-> text,
@@ -1720,7 +1722,7 @@ Prima::Dialog::FileDialog - File system related widgets and dialogs.
 
 =for podview <img src="filedlg.gif">
 
-=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/filedlg.gif">
+=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/Dialog/filedlg.gif">
 
 =head1 DESCRIPTION
 
