@@ -103,7 +103,12 @@ sub matrix
 {
 	my ( $self, @m ) = @_;
 	if ( (1 == @m) && ref($m[0]) ) {
-		$self->cmd( matrix => @{$m[0]} );
+		my $matrix = $m[0];
+		if ( UNIVERSAL::isa($matrix, 'Prima::Matrix')) {
+			$self->cmd( matrix => @{$matrix->get} );
+		} else {
+			$self->cmd( matrix => @$matrix );
+		}
 	} elsif ( @m == 6 ) {
 		$self->cmd( matrix => @m );
 	} else {
@@ -244,6 +249,7 @@ sub arc
 	@_ > 5 or Carp::croak('bad parameters to arc');
 	my ( $cx, $cy, $dx, $dy, $from, $to, $tilt) = @_;
 	return $self if $from == $to;
+	$dx--, $dy-- unless $self->{subpixel};
 	if ( $tilt // 0.0 ) {
 		return $self-> save->
 			scale( $dx / 2, $dy / 2)->
@@ -265,6 +271,7 @@ sub rarc
 	@_ > 3 or Carp::croak('bad parameters to arcto');
 	my ( $dx, $dy, $from, $to, $tilt) = @_;
 	return $self if $from == $to;
+	$dx--, $dy-- unless $self->{subpixel};
 	$self->save;
 	$self->scale( $dx / 2, $dy / 2);
 	$self->rotate( $tilt // 0.0);
@@ -278,6 +285,7 @@ sub ellipse
 	@_ > 2 or Carp::croak('bad parameters to ellipse');
 	my ( $cx, $cy, $dx, $dy, $tilt) = @_;
 	$dy //= $dx;
+	$dx--, $dy-- unless $self->{subpixel};
 	$self-> save->
 		matrix( $dx / 2, 0, 0, $dy / 2, $cx, $cy )->
 		rotate( $tilt // 0.0)->
@@ -290,6 +298,7 @@ sub chord
 	my $self = shift;
 	@_ == 6 or Carp::croak('bad parameters to chord');
 	my ( $cx, $cy, $dx, $dy, $start, $end) = @_;
+	$dx--, $dy-- unless $self->{subpixel};
 	$self-> save->
 		matrix( $dx / 2, 0, 0, $dy / 2, $cx, $cy )->
 		circular_arc( $start, $end )->
@@ -321,6 +330,7 @@ sub sector
 	my $self = shift;
 	@_ == 6 or Carp::croak('bad parameters to sector');
 	my ( $cx, $cy, $dx, $dy, $start, $end) = @_;
+	$dx--, $dy-- unless $self->{subpixel};
 	$self-> save->
 		matrix( $dx / 2, 0, 0, $dy / 2, $cx, $cy )->
 		line(0,0)->
@@ -338,7 +348,7 @@ sub round_rect
 	$dx = $maxd if $dx > $maxd;
 	$dy = $maxd if $dy > $maxd;
 	my $d = ( $dx < $dy ) ? $dx : $dy;
-	my $r = $self->{subpixel} ? $d/2 : int($d/2);
+	my $r = $self->{subpixel} ? $d/2 : ((int($d/4) * 2) + 1);
 #  plots roundrect:
 # A'        B'
 #  /------\
@@ -454,7 +464,7 @@ sub _save
 sub _restore
 {
 	my $self = shift;
-	$self->{curr} = pop @{ $self->{stack} } or die "stack undeflow";
+	$self->{curr} = pop @{ $self->{stack} } or die "stack underflow";
 }
 
 sub _set
@@ -599,6 +609,7 @@ sub _arc
 	$xopt{precision} = $self->{curr}->{precision} // 24;
 	$xopt{precision} = $scaling if $xopt{precision} // 24 > $scaling;
 	$xopt{integer}   = !$self->{subpixel};
+	$xopt{precision} *= 2 if $self->{subpixel} && $self->canvas->get_paint_state == ps::Disabled; # emulated AA for images
 	for my $set ( @$nurbset ) {
 		my ( $points, @options ) = @$set;
 		my $poly = $self-> matrix_apply( $points );
@@ -709,6 +720,7 @@ sub flatten
 			my %xopt;
 			$xopt{precision} = $self->{curr}->{precision} if defined $self->{curr}->{precision};
 			$xopt{integer}   = !$self->{subpixel};
+			$xopt{precision} *= 2 if $self->{subpixel} && $self->canvas->get_paint_state == ps::Disabled; # emulated AA for images
 			my $polyline;
 			my $nurbset = $self->arc2nurbs( $from, $to);
 			for my $set ( @$nurbset ) {
@@ -1254,9 +1266,9 @@ sub widen_new
 			} elsif ( $cmd eq 'arc') {
 				$dst->$cmd( @$param );
 			} elsif ( $cmd eq 'conic') {
-				$dst->spline( $param, degree => 2 );
+				$dst->spline( $param, degree => 2, closed => 0 );
 			} elsif ( $cmd eq 'cubic') {
-				$dst->spline( $param, degree => 3 );
+				$dst->spline( $param, degree => 3, closed => 0 );
 			} elsif ( $cmd eq 'open') {
 				$dst->open;
 			} else {
@@ -1324,7 +1336,7 @@ sub _debug_commands
 	for ( my $i = 0; $i < @$c; ) {
 		my ($cmd,$len) = @$c[$i,$i+1];
 		my @p = @$c[$i+2..$i+$len+1];
-		@p = map { 'ARRAY' eq ref ? "[@$_]" : $_ } @p;
+		@p = map { 'ARRAY' eq ref($_) ? "[@$_]" : $_ } @p;
 		print STDERR ".$cmd(@p)\n";
 		$i += $len + 2;
 	}
