@@ -7,6 +7,14 @@ use Prima;
 use Prima::Drawable::Glyphs;
 use base qw(Prima::Widget Prima::Widget::MouseScroller Prima::Widget::UndoActions Prima::Widget::BidiInput);
 
+{
+my %RNT = (
+	%{Prima::Dialog-> notification_types()},
+	Validate => nt::Notification,
+);
+
+sub notification_types { return \%RNT; }
+}
 
 sub profile_default
 {
@@ -25,7 +33,7 @@ sub profile_default
 		cursorSize     => [ Prima::Application-> get_default_cursor_width, $font-> { height}],
 		firstChar      => 0,
 		dndAware       => 'Text',
-		height         => 2 + $font-> { height} + 2,
+		height         => 4 + $font-> { height} + 2,
 		insertMode     => 0,
 		maxLen         => 256,  # length $def{ text},
 		passwordChar   => '*',
@@ -106,6 +114,15 @@ sub init
 	return %profile;
 }
 
+sub skin
+{
+	return $_[0]->SUPER::skin unless $#_;
+	my $self = shift;
+	$self->SUPER::skin($_[1]);
+	$self->borderWidth(1) if $self->SUPER::skin eq 'flat';
+	$self->repaint;
+}
+
 sub on_paint
 {
 	my ($self,$canvas) = @_;
@@ -122,11 +139,21 @@ sub on_paint
 	@selClr = ($self-> hiliteColor, $self-> hiliteBackColor);
 
 	my $border = $self-> {borderWidth};
-	$self-> rect_bevel( $canvas, Prima::rect->new(@size)->inclusive,
-		width  => $self-> {borderWidth},
-		panel  => 1,
-		fill   => $clr[1],
-	);
+	if ( $self-> skin eq 'flat') {
+		$canvas-> graphic_context( sub {
+			$canvas-> lineWidth(1);
+			$canvas-> color( $self-> dark3DColor );
+			$canvas-> rectangle(0,0,$size[0]-1,$size[1]-1);
+			$canvas-> color( $clr[1]);
+			$canvas-> bar( 1, 1, $size[0]-2, $size[1]-2);
+		});
+	} else {
+		$self-> rect_bevel( $canvas, Prima::rect->new(@size)->inclusive,
+			width => $border,
+			panel => 1,
+			fill  => $clr[1],
+		);
+	}
 
 	return if $size[0] <= $border * 2 + 2;
 	my $cap = $self-> {glyphs} or return;
@@ -242,6 +269,7 @@ sub set_text
 	$cap = substr( $cap, 0, $self-> {maxLen})
 		if $self-> {maxLen} >= 0 and length($cap) > $self-> {maxLen};
 
+	$self-> notify(q(Validate), \$cap);
 	$self-> SUPER::set_text($cap);
 
 	$cap = $self-> {passwordChar} x length $cap if $self-> {writeOnly};
@@ -331,7 +359,7 @@ sub handle_input
 	$self-> charOffset(
 		$self->{glyphs}->index2cluster(
 			$new_offset,
-			$opt{action} =~ /^(insert|overtype)$/
+			($opt{action} eq 'insert')
 		)
 	) if defined $new_offset;
 }
@@ -579,8 +607,8 @@ sub paste
 	my $s = $::application-> Clipboard-> text;
 	return if !defined($s) or length( $s) == 0;
 
-	my ($p_start, $p_end) = ($start == $end) ? 
-		(($self->{glyphs}->cursor2offset($self->charOffset, $self->textDirection)) x 2) :
+	my ($p_start, $p_end) = ($start == $end) ?
+		(($self->cursor2text_offset($self->charOffset)) x 2) :
 		$self->selection_strpos;
 	substr( $cap, $p_start, $p_end - $p_start) = $s;
 	$self-> selection(0,0);
@@ -888,6 +916,12 @@ sub fit_clusters_back
 	return $sub-> sub_text_wrap( $self, 0, undef, $w, tw::ReturnFirstLineLength);
 }
 
+sub cursor2text_offset
+{
+	my ( $self, $cursor ) = @_;
+	return $self->{glyphs}->cursor2offset($cursor, $self->textDirection);
+}
+
 sub set_char_offset
 {
 	my ( $self, $offset) = @_;
@@ -897,8 +931,6 @@ sub set_char_offset
 	$offset = 0 if $offset < 0;
 	return if $self-> {charOffset} == $offset;
 
-	$self->{glyphs}->cursor2offset($offset, $self->textDirection);
-
 	$self-> push_undo_action( 'charOffset', $offset) unless $self->has_undo_action('charOffset');
 
 	my $border = $self-> {borderWidth};
@@ -1153,6 +1185,30 @@ sub selText    {
 	} : return substr( $_[ 0]-> text, $f, $t - $f);
 }
 
+#sub on_validate
+#{
+#	my ($self, $textref) = @_;
+#}
+
+sub blink
+{
+	my ($self, %opt) = @_;
+	return if $self-> bring('BlinkTimer');
+	my $bc = $self-> backColor;
+	my $c  = $self-> color;
+	$self-> color($opt{color} // cl::White);
+	$self-> backColor($opt{backColor} // 0xff8080);
+	$self-> insert( Timer =>
+		name    => 'BlinkTimer',
+		timeout => 300,
+		onTick  => sub {
+			$_[0]-> destroy;
+			$self-> color($c);
+			$self-> backColor($bc);
+		},
+	)-> start;
+}
+
 1;
 
 =pod
@@ -1189,6 +1245,14 @@ contained in L<text> property.
 The notification is called when the L<text> property is changed, either
 interactively or as a result of direct call.
 
+=item Validate TEXT_REF
+
+The notification is called right before the L<text> property is changed, either
+interactively or as a result of direct call. The custom code has a chance to
+validate the text and/or provide some sort of interactive feedback.
+
+See also: L</blink>
+
 =back
 
 =head2 Properties
@@ -1314,6 +1378,12 @@ Default value: 0
 
 =over
 
+=item blink %options
+
+Produces a short blink by setting background to red color.
+Can be used to signal invalid input, f ex from C<on_validate>.
+C<%options> allows C<backColor> and C<color> entries.
+
 =item copy
 
 Copies the selected text, if any, to the clipboard.
