@@ -463,9 +463,9 @@ sub init
 	$self->{selectedItem} = -1;
 	for ( qw( borderSize ))
 		{ $self-> {$_} = 1; }
+	$self->{lock_reset} = 1;
 	my %profile = $self-> SUPER::init(@_);
 
-	$self->{lock_reset} = 1;
 	for ( qw( borderSize parent parentIndex ))
 		{ $self-> $_( $profile{ $_}); }
 	delete $self->{lock_reset};
@@ -473,6 +473,15 @@ sub init
 	return %profile;
 }
 
+sub skin
+{
+	return $_[0]->SUPER::skin unless $#_;
+	my $self = shift;
+	$self->SUPER::skin($_[1]);
+	$self->borderSize(1) if $self->SUPER::skin eq 'flat';
+	$self->repaint;
+}
+
 sub borderSize
 {
 	return $_[0]->{borderSize} unless $#_;
@@ -578,6 +587,7 @@ sub enter_submenu
 	}
 
 	$self->notify(qw(Submenu), $i);
+	return unless $self->alive;
 
 	$self->{submenu_index} = $i;
 
@@ -642,7 +652,9 @@ sub on_keydown
 
 	my $submenu = $self;
 	my $level = 0;
+	my @hierarchy;
 	while ($submenu->{submenu}) {
+		push @hierarchy, $submenu;
 		$submenu = $submenu->{submenu};
 		$level++;
 	}
@@ -689,8 +701,10 @@ sub on_keydown
 	} elsif ( $key == kb::Enter ) {
 		$submenu-> execute_selected;
 		$ok = 1;
+		return unless $self->alive;
 	}
 
+	# shortcuts
 	my $m = $self->menu;
 	if ( defined( my $itemid = $m->find_item_by_key(
 		$m->translate_key($code, $key, $mod)
@@ -701,11 +715,32 @@ sub on_keydown
 		}
 	}
 
-	# XXX immediate ~ hotkeys
+	# immediate ~ hotkeys
+	if ( !$ok && chr($code) =~ /^[0-9a-z]$/i) {
+		MENU: for my $upper ( $submenu, reverse @hierarchy ) {
+			my $idx = -1;
+			for my $c ( @{ $upper->{cache} }) {
+				$idx++;
+				next unless defined $c->[ITEMID];
+				my $text = $m->text($c->[ITEMID]);
+				next unless defined $text;
+				next unless $text =~ m/(?<!~)~([a-z0-9])/i;
+				next unless lc(chr($code)) eq lc($1);
+				if ($self-> menu-> is_submenu($c->[ITEMID])) {
+					$upper->selectedItem($idx);
+					$upper->enter_submenu;
+				} else {
+					$self-> execute_item($c->[ITEMID]);
+				}
+				$ok = 1;
+				last MENU;
+ 			}
+		}
+	}
 
 	$ok = 1 if $skip && $level > 0;
 
-	$self->clear_event if $ok;
+	$self->clear_event if $ok && $self->alive;
 }
 
 sub on_fontchanged
@@ -734,10 +769,19 @@ sub on_paint
 	my @sz = $self->size;
 	my $caches = $self->{cache};
 	if ( $self->{vertical} ) {
-		$self->rect_bevel( $canvas, 0, 0, $sz[0]-1, $sz[1]-1,
-			width => $self->{borderSize}, 
-			fill => $c[ci::Normal],
-		);
+		if ( $self-> skin eq 'flat') {
+			$canvas-> graphic_context( sub {
+				$canvas-> color( $self-> dark3DColor ); 
+				$canvas-> rectangle(0, 0, $sz[0]-1, $sz[1]-1);
+				$canvas-> color( $c[ci::Normal] );
+				$canvas-> bar( 1, 1, $sz[0]-2, $sz[1]-2);
+			});
+		} else {
+			$self->rect_bevel( $canvas, 0, 0, $sz[0]-1, $sz[1]-1,
+				width => $self->{borderSize}, 
+				fill => $c[ci::Normal],
+			);
+		}
 	} else {
 		$self->clear;
 	}
