@@ -4,14 +4,103 @@
 #   Outline
 #   DirectoryOutline
 package Prima::Outlines;
-
+our %images;
+our $default_style = 'plusminus';
 use strict;
 use warnings;
 use Cwd;
 use Prima qw(StdBitmap);
 
+sub init_image_plusminus
+{
+	return if $images{plusminus};
+
+	my $i = 0;
+	my $uis = $::application->uiScaling;
+	my $iw = int( $uis * 11 + .5);
+	$iw++ unless $iw % 2;
+	my @imageSize = ($iw, $iw);
+	my $xd = int(2 * $uis + .5);
+	my $lw = int( $uis + .5);
+	$lw-- if $lw > 1 && $lw % 2;
+	my @c = map { int ( $_ / 2 ) } @imageSize;
+
+	my @images;
+	for my $i (0,1) {
+		$images[$i] = Prima::DeviceBitmap->new(
+			size      => \@imageSize,
+			backColor => cl::White,
+			color     => cl::Gray,
+		);
+		$images[$i]->clear;
+		$images[$i]->lineWidth($lw);
+		$images[$i]->lineEnd(le::Square);
+		$images[$i]->rectangle(0,0,$imageSize[0]-1,$imageSize[1]-1);
+		$images[$i]->color(cl::Black);
+		$images[$i]->line( $xd, $c[1], $imageSize[0]-$xd-1, $c[1] );
+	}
+	$images[1]->line( $c[0], $xd, $c[0], $imageSize[1]-$xd-1);
+
+	$images{plusminus}->{expanded}  = $images[0];
+	$images{plusminus}->{collapsed} = $images[1];
+}
+
+sub init_image_triangle
+{
+	return if $images{plusminus};
+
+	my $i = 0;
+	my $uis = $::application->uiScaling;
+	my $iw = int( $uis * 11 + .5);
+	$iw++ unless $iw % 2;
+	my @imageSize = ($iw, $iw);
+	my $lw = int( $uis + .5);
+	$lw-- if ($lw > 1 && $lw % 2) || !$::application->can_draw_alpha;
+	my @c2 = map { int ( $_ / 2 ) } @imageSize;
+
+	my @images;
+	for my $i (0,1) {
+		$images[$i] = Prima::DeviceBitmap->new(
+			size      => \@imageSize,
+			backColor => cl::White,
+			color     => cl::Black,
+		);
+		$images[$i]->clear;
+		$images[$i]->lineWidth($lw);
+		$images[$i]->lineJoin(lj::Miter);
+		$images[$i]->translate(0, $imageSize[1]*0.25) if $i;
+		$images[$i]->polyline( [
+			$i ? (
+				$lw, $c2[1], $imageSize[0]-1-$lw, $c2[1],
+				$c2[0], $lw, $lw, $c2[1]
+			) : (
+				$c2[0], $lw, $c2[0], $imageSize[1]-1-$lw,
+				$imageSize[0]-1-$lw, $c2[1], $c2[0], $lw
+			)
+		]);
+	}
+
+	$images{triangle}->{collapsed} = $images[0];
+	$images{triangle}->{expanded}  = $images[1];
+}
+
+sub icon
+{
+	shift if @_ % 2;
+	my ( %opt ) = @_;
+
+	my $type  = $opt{type} // 'collapsed';
+	my $style = $opt{style} // 'default';
+
+	$style = $default_style if $style eq 'default';
+	my $meth = 'init_image_' . $style;
+	$meth = 'init_image_plusminus' unless __PACKAGE__->can($meth);
+	__PACKAGE__->$meth();
+
+	return $images{$style}->{$type};
+}
+
 package Prima::OutlineViewer;
-use vars qw(@images @imageSize);
 use base qw(Prima::Widget Prima::Widget::MouseScroller Prima::Widget::GroupScroller Prima::Widget::ListBoxUtils);
 
 use constant DATA     => 0;
@@ -54,8 +143,12 @@ sub profile_default
 		borderWidth    => 2,
 		extendedSelect => 0,
 		dragable       => 1,
+		drawLines      => undef,
 		hScroll        => 0,
 		focusedItem    => -1,
+		iconCollapsed  => undef,
+		iconExpanded   => undef,
+		iconStyle      => 'default',
 		indent         => int( 12 * $::application->uiScaling + .5),
 		itemHeight     => $def-> {font}-> {height},
 		items          => [],
@@ -75,31 +168,6 @@ sub profile_default
 	return $def;
 }
 
-sub init_images
-{
-	return if @images;
-
-	my $i = 0;
-	my $uis = $::application->uiScaling;
-	@imageSize = map { $uis * $_ } (11,11);
-	my $xd = int(2 * $uis + .5);
-	my $lw = int( $uis + .5);
-	my @c = map { $_ / 2 } @imageSize;
-	for my $i (0,1) {
-		$images[$i] = Prima::DeviceBitmap->new(
-			size      => \@imageSize,
-			backColor => cl::White,
-			color     => cl::Gray,
-		);
-		$images[$i]->clear;
-		$images[$i]->lineWidth($lw);
-		$images[$i]->lineEnd(le::Square);
-		$images[$i]->rectangle(0,0,$imageSize[0]-$lw%2,$imageSize[1]-$lw%2);
-		$images[$i]->color(cl::Black);
-		$images[$i]->line( $xd, $c[1], $imageSize[0]-$xd-1, $c[1] );
-	}
-	$images[1]->line( $c[0], $xd, $c[0], $imageSize[1]-$xd-1);
-}
 
 sub profile_check_in
 {
@@ -121,7 +189,6 @@ use constant STACK_FRAME => 64;
 sub init
 {
 	my $self = shift;
-	init_images;
 	for ( qw( topItem focusedItem))
 		{ $self-> {$_} = -1; }
 	for ( qw( autoHScroll autoVScroll scrollTransaction dx dy hScroll vScroll
@@ -131,11 +198,14 @@ sub init
 	for ( qw( itemHeight indent))
 		{ $self-> {$_} = 1; }
 	$self-> {items}      = [];
+	$self-> {iconStyle}  = 'default';
 	my %profile = $self-> SUPER::init(@_);
 	$self-> setup_indents;
 	$self->{$_} = $profile{$_} for qw(scrollBarClass hScrollBarProfile vScrollBarProfile);
 	for ( qw( autoHScroll autoVScroll hScroll vScroll offset itemHeight autoHeight borderWidth
-		indent items focusedItem topItem showItemHint dragable multiSelect extendedSelect))
+		indent items focusedItem topItem showItemHint dragable multiSelect extendedSelect
+		iconStyle skin iconCollapsed iconExpanded drawLines
+	))
 		{ $self-> $_( $profile{ $_}); }
 	$self-> reset;
 	$self-> reset_scrolls;
@@ -264,9 +334,10 @@ sub on_paint
 	my @clr    = $self-> enabled ?
 	( $self-> color, $self-> backColor) :
 	( $self-> disabledColor, $self-> disabledBackColor);
-	my ( $ih, $iw, $indent, $foc, @a) = (
-		$self-> { itemHeight}, $self-> { maxWidth},
+	my ( $ih, $iw, $lines, $indent, $foc, @a) = (
+		$self-> { itemHeight}, $self-> { maxWidth}, $self->{drawLines},
 		$self-> {indent}, $self-> {focusedItem}, $self-> get_active_area( 1, @size));
+	my @images = @{ $self }{qw(iconExpanded iconCollapsed) };
 	my $i;
 	my $j;
 	my $locWidth = $a[2] - $a[0] + 1;
@@ -297,8 +368,8 @@ sub on_paint
 	$lastItem     = $self-> {count} - 1 if $lastItem > $self-> {count} - 1;
 	my $firstY    = $a[3] + 1 + $ih * $topItem;
 	my $lineY     = $a[3] + 1 - $ih * ( 1 + $timin - $topItem);
-	my $dyim      = int(( $ih - $imageSize[1]) / 2) + 1;
-	my $dxim      = int( $imageSize[0] / 2);
+	my $dyim      = int(( $ih - $self->{imageSize}->[1]) / 2) + 1;
+	my $dxim      = int( $self->{imageSize}->[0] / 2);
 
 # drawing lines
 	my @lines;
@@ -311,8 +382,7 @@ sub on_paint
 		color       => cl::White,
 		backColor   => cl::Black,
 		rop2        => rop::CopyPut,
-	);
-
+	) if $lines;
 
 	my ($array, $idx, $lim, $level) = ([['root'],$self-> {items}], 0, scalar @{$self-> {items}}, 0);
 	my @stack;
@@ -357,20 +427,22 @@ sub on_paint
 			if ( defined $lines[ $level]) {
 				$canvas-> bar(
 					$l, $firstY - $ih * $lines[ $level],
-					$l, $firstY - $ih * ( $position + 0.5))
-				if $position >= $timin;
+					$l, $firstY - $ih * ( $position + 0.5)
+				) if $lines and $position >= $timin;
 				$lines[ $level] = undef;
 			} elsif ( $position > 0) {
 				# first and last
 				$canvas-> bar(
 					$l, $firstY - $ih * ( $position - 0.5),
-					$l, $firstY - $ih * ( $position + 0.5))
+					$l, $firstY - $ih * ( $position + 0.5)
+				) if $lines;
 			}
 		} elsif ( !defined $lines[$level]) {
 			$lines[$level] = $position ? $position - 0.5 : 0.5;
 		}
 		if ( $position >= $timin) {
-			$canvas-> bar( $l + 1, $lineY + $ih/2, $l + $indent - 1, $lineY + $ih/2);
+			$canvas-> bar( $l + 1, $lineY + $ih/2, $l + $indent - 1, $lineY + $ih/2)
+				if $lines;
 			if ( defined $node-> [DOWN]) {
 				my $i = $images[($node-> [EXPANDED] == 0) ? 1 : 0];
 				push( @marks, [$l - $dxim, $lineY + $dyim, $i]) if $i;
@@ -405,19 +477,21 @@ sub on_paint
 	}}
 
 # drawing line ends
-	$i = 0;
-	for ( @lines) {
-		$i++;
-		next unless defined $_;
-		my $l = ( $i - 0.5) * $indent + $deltax;;
-		$canvas-> bar( $l, $firstY - $ih * $_, $l, 0);
+	if ( $lines ) {
+		$i = 0;
+		for ( @lines) {
+			$i++;
+			next unless defined $_;
+			my $l = ( $i - 0.5) * $indent + $deltax;;
+			$canvas-> bar( $l, $firstY - $ih * $_, $l, 0);
+		}
+		$canvas-> set(
+			fillPattern => fp::Solid,
+			color       => $clr[0],
+			backColor   => $clr[1],
+			rop2        => rop::NoOper,
+		);
 	}
-	$canvas-> set(
-		fillPattern => fp::Solid,
-		color       => $clr[0],
-		backColor   => $clr[1],
-		rop2        => rop::NoOper,
-	);
 
 #
 	$canvas-> put_image( @$_) for @marks;
@@ -471,8 +545,8 @@ sub on_mousedown
 
 	if (
 		$rec &&
-		( $x >= ( 1 + $lev) * $i + $a[0] - $o - $imageSize[0] / 2) &&
-		( $x <  ( 1 + $lev) * $i + $a[0] - $o + $imageSize[0] / 2)
+		( $x >= ( 1 + $lev) * $i + $a[0] - $o - $self->{imageSize}->[0] / 2) &&
+		( $x <  ( 1 + $lev) * $i + $a[0] - $o + $self->{imageSize}->[0] / 2)
 	) {
 		$self-> adjust( $item, $rec-> [2] ? 0 : 1) if $rec-> [1];
 		return;
@@ -518,8 +592,8 @@ sub on_mouseclick
 	my ( $rec, $lev) = $self-> get_item( $item);
 	if (
 		$rec &&
-		( $x >= ( 1 + $lev) * $i + $self-> {indents}-> [0] - $o - $imageSize[0] / 2) &&
-		( $x <  ( 1 + $lev) * $i + $self-> {indents}-> [0] - $o + $imageSize[0] / 2)
+		( $x >= ( 1 + $lev) * $i + $self-> {indents}-> [0] - $o - $self->{imageSize}->[0] / 2) &&
+		( $x <  ( 1 + $lev) * $i + $self-> {indents}-> [0] - $o + $self->{imageSize}->[0] / 2)
 	) {
 		$self-> adjust( $item, $rec-> [EXPANDED] ? 0 : 1) if $rec-> [DOWN];
 		return;
@@ -1331,6 +1405,53 @@ sub dragable
 	$_[0]-> {dragable} = $_[1];
 }
 
+sub drawLines
+{
+	return $_[0]-> {drawLines} unless $#_;
+	return unless defined $_[1];
+	$_[0]-> {drawLines} = $_[1];
+	$_[0]-> repaint;
+}
+
+sub iconCollapsed
+{
+	return $_[0]-> {iconCollapsed} unless $#_;
+	return unless $_[1];
+	$_[0]-> {iconCollapsed} = $_[1];
+	$_[0]-> {imageSize} = [$_[1]->size];
+	$_[0]-> repaint;
+}
+
+sub iconExpanded
+{
+	return $_[0]-> {iconExpanded} unless $#_;
+	return unless $_[1];
+	$_[0]-> {iconExpanded} = $_[1];
+	$_[0]-> repaint;
+}
+
+sub iconStyle
+{
+	return $_[0]-> {iconStyle} unless $#_;
+	my ( $self, $style ) = @_;
+	$self->iconCollapsed( Prima::Outlines->icon( type => 'collapsed', style => $style) );
+	$self->iconExpanded(  Prima::Outlines->icon( type => 'expanded' , style => $style) );
+	$self->drawLines( $style ne 'triangle' );
+}
+
+sub skin
+{
+	return $_[0]->SUPER::skin unless $#_;
+	my $self = shift;
+	$self->SUPER::skin($_[1]);
+	my $skin = $self->SUPER::skin;
+	if ($skin eq 'flat') {
+		$self->iconStyle('triangle');
+	} elsif ( $skin eq 'classic') {
+		$self->iconStyle('plusminus');
+	}
+	$self->repaint;
+}
 
 sub get_index
 {
@@ -2101,6 +2222,15 @@ Prima::Outlines - tree view widgets
 
 =for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/outline.gif">
 
+	my $outline = Prima::StringOutline-> create(
+		iconStyle => 'triangle',
+		...
+	);
+
+=for podview <img src="outline2.gif">
+
+=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/outline2.gif">
+
 =head1 DESCRIPTION
 
 The module provides a set of widget classes, designed to display a tree-like
@@ -2179,6 +2309,12 @@ together with left mouse button. If 0, item dragging is disabled.
 
 Default value: 1
 
+=item drawLines BOOLEAN
+
+If 1, draws dotted tree lines left to the items.
+
+Default value: 1
+
 =item extendedSelect BOOLEAN
 
 Regards the way the user selects multiple items and is only actual
@@ -2196,6 +2332,20 @@ It is mostly a run-time property, however, it can be set
 during the widget creation stage given that the item list is
 accessible on this stage as well.
 
+=item iconCollapsed ICON
+
+Sets the image that is to be displayed when a tree branch is collapsed
+
+=item iconExpanded ICON
+
+Sets the image that is to be displayed when a tree branch is expanded
+
+=item iconStyle STYLE
+
+Sets visual style, one of: C<default>, C<plusminus>, C<triangle>.
+
+The default style is set in C<$Prima::Outlines::default_style> and is currently 'plusminus'
+
 =item indent INTEGER
 
 Width in pixels of the indent between item levels.
