@@ -97,7 +97,7 @@ sub change_transform
 
 	$self-> emit_content("h @pm re W n");
 
-	my $m  = $self-> matrix;
+	my $m = $self-> get_matrix;
 	if ( $self->{reversed}) {
 		$$m[5] -= ($self->point2pixel($pm[2]))[0];
 		$m->rotate(90);
@@ -816,8 +816,8 @@ sub fillPattern
 	my ($solidBack, $solidFore) = (0,0);
 	my $xid;
 	if( (ref($fp) // '') eq 'ARRAY') {
-		$solidBack = ! grep { $_ != 0    } @$fp;
-		$solidFore = ! grep { $_ != 0xff } @$fp;
+		$solidBack = fp::is_empty($fp);
+		$solidFore = fp::is_solid($fp);
 		if ( !$solidBack && !$solidFore) {
 			my $fpid = join( '', map { sprintf("%02x", $_)} @$fp);
 			$xid  = $self-> emit_pattern($fpid, $fp);
