@@ -1,6 +1,8 @@
 use strict;
 use warnings;
-use Prima qw(StdDlg Notebooks MsgBox ComboBox FontDialog ColorDialog IniFile Utils RubberBand KeySelector);
+use Prima qw(Notebooks MsgBox ComboBox
+Dialog::FontDialog Dialog::ColorDialog Dialog::FileDialog Dialog::ImageDialog
+IniFile Utils Widget::RubberBand KeySelector Utils sys::GUIException);
 use Prima::VB::VBLoader;
 use Prima::VB::VBControls;
 use Prima::VB::CfgMaint;
@@ -53,7 +55,7 @@ my $fontDlg;
 sub open_dialog
 {
 	my %profile = @_;
-	$openFileDlg = Prima::OpenDialog-> create(
+	$openFileDlg = Prima::Dialog::OpenDialog-> create(
 		icon => $VB::ico,
 		directory => $VB::main-> {ini}-> {OpenPath},
 		system => 1,
@@ -65,7 +67,7 @@ sub open_dialog
 sub save_dialog
 {
 	my %profile = @_;
-	$saveFileDlg = Prima::SaveDialog-> create(
+	$saveFileDlg = Prima::Dialog::SaveDialog-> create(
 		icon => $VB::ico,
 		directory => $VB::main-> {ini}-> {SavePath},
 		system => 1,
@@ -77,7 +79,7 @@ sub save_dialog
 sub image_open_dialog
 {
 	my %profile = @_;
-	$openImageDlg = Prima::ImageOpenDialog-> create(
+	$openImageDlg = Prima::Dialog::ImageOpenDialog-> create(
 		icon => $VB::ico,
 		directory => $VB::main-> {ini}-> {OpenPath},
 	) unless $openImageDlg;
@@ -88,7 +90,7 @@ sub image_open_dialog
 sub image_save_dialog
 {
 	my %profile = @_;
-	$saveImageDlg = Prima::ImageSaveDialog-> create(
+	$saveImageDlg = Prima::Dialog::ImageSaveDialog-> create(
 		icon => $VB::ico,
 		directory => $VB::main-> {ini}-> {SavePath},
 	) unless $saveImageDlg;
@@ -99,7 +101,7 @@ sub image_save_dialog
 sub font_dialog
 {
 	my %profile = @_;
-	$fontDlg = Prima::FontDialog-> create(
+	$fontDlg = Prima::Dialog::FontDialog-> create(
 		icon => $VB::ico,
 		name => 'Select font',
 	) unless $fontDlg;
@@ -110,7 +112,7 @@ sub font_dialog
 sub accelItems
 {
 	return [
-		['openitem' => '~Open' => 'Ctrl+O' => '^O' => sub { $VB::main-> open;}],
+		['openitem' => '~Open' => 'Ctrl+O' => '^O' => sub { $VB::main-> open_file;}],
 		['-saveitem1' => '~Save' => 'Ctrl+S' => '^S' => sub {$VB::main-> save;}],
 		['Exit' => 'Ctrl+Q' => '^Q' => sub{ $VB::main-> close;}],
 		['Object Inspector' => 'F11' => 'F11' => sub { $VB::main-> bring_inspector; }],
@@ -691,9 +693,11 @@ sub on_paint
 	my ( $self, $canvas) = @_;
 	$canvas-> backColor( $self-> backColor);
 	$canvas-> color( cl::Blue);
+	$canvas-> rop2(rop::CopyPut);
 	$canvas-> fillPattern([0,0,0,0,4,0,0,0]);
 	my @sz = $canvas-> size;
 	$canvas-> bar(0,0,@sz);
+	$canvas-> rop2(rop::NoOper);
 	$canvas-> fillPattern( fp::Solid);
 	$canvas-> linePattern( lp::Dash);
 	$canvas-> line( $self-> {guidelineX}, 0, $self-> {guidelineX}, $sz[1]);
@@ -935,9 +939,7 @@ sub fm_reclass
 	my $self = $VB::form;
 	return unless $self;
 	my @wijs = $VB::form-> marked_widgets;
-	return unless scalar @wijs;
-	my $cx = $wijs[0];
-	$self = $cx if $cx;
+	$self = $wijs[0] if @wijs;
 	my $lab_text = 'Class name';
 	$lab_text =  "Temporary class for ".$self-> {realClass} if defined $self-> {realClass};
 	my $dlg = Prima::Dialog-> create(
@@ -1187,6 +1189,7 @@ data =>
 "\x07\x00\x00\x00\x02\x00\x00\x00\x02\x00\x00\x00".
 ''),
 		size => [ 29, 130],
+		flat    => 1,
 		onClick => sub {
 			my $i  = $d-> Items;
 			my $fi = $i-> focusedItem;
@@ -1206,6 +1209,7 @@ data =>
 "\x7f\xf0\x00\x00\x7f\xf0\x00\x00\xff\xf8\x00\x00".
 ''),
 		size => [ 29, 130],
+		flat => 1,
 		onClick => sub {
 			my $i   = $d-> Items;
 			my $fi  = $i-> focusedItem;
@@ -1252,6 +1256,7 @@ package Prima::VB::MainPanel;
 use strict;
 use vars qw(@ISA *do_layer);
 @ISA = qw(Prima::Window);
+use Prima::sys::FS;
 
 sub profile_default
 {
@@ -1270,7 +1275,7 @@ sub profile_default
 		menuItems      => [
 			['~File' => [
 				['newitem' => '~New' => 'Ctrl+N' => '^N' =>     sub {$_[0]-> new;}],
-				['openitem' => '~Open' => 'Ctrl+O' => '^O' =>   sub {$_[0]-> open;}],
+				['openitem' => '~Open' => 'Ctrl+O' => '^O' =>   sub {$_[0]-> open_file;}],
 				['-saveitem1' => '~Save' => 'Ctrl+S' => '^S' => sub {$_[0]-> save;}],
 				['-saveitem2' =>'Save ~as...' =>                sub {$_[0]-> saveas;}],
 				['closeitem' =>'~Close' => 'Ctrl+W' => '^W' =>  sub { $VB::form-> close if $VB::form}],
@@ -1368,6 +1373,7 @@ sub init
 		hint      => 'New',
 		image     => $images[0],
 		glyphs    => 2,
+		flat      => 1,
 		onClick   => sub { $VB::main-> new; } ,
 	);
 
@@ -1377,7 +1383,8 @@ sub init
 		hint      => 'Open',
 		image     => $images[1],
 		glyphs    => 2,
-		onClick   => sub { $VB::main-> open; } ,
+		flat      => 1,
+		onClick   => sub { $VB::main-> open_file; } ,
 	);
 
 	$self-> {savebutton} = $self-> insert( SpeedButton =>
@@ -1386,6 +1393,7 @@ sub init
 		hint      => 'Save',
 		image     => $images[2],
 		glyphs    => 2,
+		flat      => 1,
 		onClick   => sub { $VB::main-> save; } ,
 	);
 
@@ -1395,6 +1403,7 @@ sub init
 		hint      => 'Run',
 		image     => $images[3],
 		glyphs    => 2,
+		flat      => 1,
 		onClick   => sub { $VB::main-> form_run} ,
 	);
 
@@ -1428,17 +1437,7 @@ sub init
 		origin     => [$s*12,1],
 		size       => [$self-> {nb}-> width-$s*24,36*$s+8],
 		growMode   => gm::Floor,
-		backColor  => cl::Gray,
 		name       => 'NBPanel',
-		onPaint    => sub {
-			my ( $self, $canvas) = @_;
-			my @sz = $self-> size;
-			$canvas-> rect3d(0,0,$sz[0]-1,$sz[1]-1,
-				1,cl::Black,cl::Black,$self-> backColor);
-			my $i = 0;
-			my $sz = $s * 36;
-			$canvas-> rectangle($i-$sz-2,2,$i,$sz+4) while (($i+=$sz+4)<($sz[0]+$sz));
-		},
 	);
 
 	$self-> {leftScroll} = $self-> {nb}-> insert( SpeedButton =>
@@ -1446,11 +1445,12 @@ sub init
 		size    => [map { $s * $_ } 11,36],
 		name    => 'LeftScroll',
 		autoRepeat => 1,
+		flat    => 1,
 		onPaint => sub {
 			$_[0]-> on_paint( $_[1]);
 			$_[1]-> color( $_[0]-> enabled ? cl::Black : cl::Gray);
 			$_[1]-> fillpoly([map { $s * $_ } 7,4,7,32,3,17]);
-		},
+		}, 
 		delegations => [ $self, qw(Click)],
 	);
 
@@ -1460,6 +1460,7 @@ sub init
 		name    => 'RightScroll',
 		growMode => gm::Right,
 		autoRepeat => 1,
+		flat    => 1,
 		onPaint => sub {
 			$_[0]-> on_paint( $_[1]);
 			$_[1]-> color( $_[0]-> enabled ? cl::Black : cl::Gray);
@@ -1587,6 +1588,7 @@ sub reset_tabs
 			hint   => $class,
 			name   => 'ClassSelector',
 			image  => $i,
+			flat   => 1,
 			origin => [ $offsets{$info{page}}, 4],
 			size   => [ map { $s * $_ } 36, 36],
 			delegations => [$self, qw(Click)],
@@ -1708,7 +1710,7 @@ sub inspect_load_data
 
 	my $fn = ( $asFile ? $data : "input data");
 	if ( $asFile) {
-		unless (CORE::open( F, "< $data")) {
+		unless (open F, "<", $data) {
 			Prima::MsgBox::message( "Error loading " . $data);
 			return;
 		}
@@ -1911,7 +1913,7 @@ sub load_file
 	$_-> notify(q(Load)) for $VB::form-> widgets;
 }
 
-sub open
+sub open_file
 {
 	my $self = $_[0];
 
@@ -2210,7 +2212,7 @@ sub save
 
 	return $self-> saveas unless defined $self-> {fmName};
 
-	if ( CORE::open( F, ">".$self-> {fmName})) {
+	if ( open F, ">", $self-> {fmName}) {
 		local $/;
 		$VB::main-> wait;
 		my $c = $asPL ? $self-> write_PL : $self-> write_form;
@@ -2427,7 +2429,7 @@ sub bring_font_dialog
 	if ( $VB::font_dialog) {
 		$VB::font_dialog-> restore if $VB::font_dialog-> windowState == ws::Minimized;
 	} else {
-		$VB::font_dialog = Prima::FontDialog-> new(
+		$VB::font_dialog = Prima::Dialog::FontDialog-> new(
 			borderIcons => bi::All & ~bi::Maximize,
 			taskListed  => 1,
 			onDestroy   => sub { $VB::font_dialog = undef },
@@ -2453,7 +2455,7 @@ sub bring_color_dialog
 	if ( $VB::color_dialog) {
 		$VB::color_dialog-> restore if $VB::color_dialog-> windowState == ws::Minimized;
 	} else {
-		$VB::color_dialog = Prima::ColorDialog-> new(
+		$VB::color_dialog = Prima::Dialog::ColorDialog-> new(
 			borderIcons => bi::All & ~bi::Maximize,
 			taskListed  => 1,
 			onDestroy => sub { $VB::color_dialog = undef },
@@ -2515,10 +2517,7 @@ $VB::main-> update_menu();
 
 $VB::main-> load_file( $ARGV[0]) if @ARGV && -f $ARGV[0] && -r _;
 
-while ($::application) {
-	eval { run Prima };
-	Prima::MsgBox::message( "$@") if $::application && $@;
-}
+run Prima;
 
 1;
 
@@ -2530,6 +2529,11 @@ __END__
 
 VB - Visual Builder for the Prima toolkit
 
+=for podview <img src="vb-large.png">
+
+=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/vb-large.png">
+
+
 =head1 DESCRIPTION
 
 Visual Builder is a RAD-style suite for designing forms under
