@@ -702,6 +702,7 @@ typedef struct _UnixGuts
 	Bool                         xft_no_antialias;
 	Bool                         randr_extension;
 	Bool                         render_extension;
+	Bool                         render_supports_argb32;
 	Bool                         composite_extension;
 	int                          composite_opcode;
 	Bool                         composite_error_triggered;
@@ -923,6 +924,7 @@ typedef struct _drawable_sys_data
 		unsigned withdrawn                : 1;
 		unsigned zoomed                   : 1;
 		unsigned xft_clip                 : 1;
+		unsigned xrender_sync             : 1;
 	} flags;
 	ImageCache image_cache;
 	Handle preexec_focus;
@@ -951,6 +953,8 @@ typedef struct _drawable_sys_data
 #define XF_IN_PAINT(x)  ((x)->flags.paint)
 #define XF_LAYERED(x)  ((x)->flags.layered)
 #define XFLUSH          if (XX->flags.force_flush) XFlush(DISP)
+#define XRENDER_SYNC    if (XX->flags.xrender_sync) XSync(DISP, XX->flags.xrender_sync = false)
+#define XRENDER_SYNC_NEEDED XX->flags.xrender_sync = true
 
 typedef struct _PaintState
 {
@@ -1181,7 +1185,7 @@ extern Bool
 prima_init_color_subsystem( char * error_buf);
 
 extern Bool
-prima_init_xrender_subsystem( char * error_buf);
+prima_init_xrender_subsystem( char * error_buf, Bool disable_argb32);
 
 extern Bool
 prima_find_color_mask_range( unsigned long mask, unsigned int * shift, unsigned int * range);
